/*  osdchat - on-screen display messaging service
 *  Copyright (c) 2003 by Mike Smelov  <mike-at-argoncorp-dot-com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  See LICENSE for details.
 *
 */

 /*  block.c */

#include "osdchatd.h"
#include "common.h"

/* defined in cs.c */
extern struct client client_list[MAX_NCLIENTS];

/* block list is a circular buffer */
static int *blocked_ips;
static int b_index;

/* initilize blocked ip list */
void cs_block_init(void)
{
	int n = MAX_IPBLOCKS * sizeof(int);

	if (blocked_ips != NULL)
		return;

	blocked_ips = (int *) xmalloc(n);
	memset(blocked_ips, 0, n);
}

/* free blocked ip list */
void cs_block_free(void)
{
	free(blocked_ips);
}

/* place client's IP in a block list */
int cs_block(int fd)
{
	int i;

	if (blocked_ips == NULL) {
		errno = EFAULT;
		return -1;
	}

	for (i = 0; i < num_clients; i++) {
		if (client_list[i].fd == fd) {
			memcpy(&blocked_ips[b_index], &client_list[i].ip, sizeof(int));

			(b_index == MAX_IPBLOCKS - 1) ? b_index = 0 : b_index++;
			return 1;
		}
	}

	errno = EBADF;
	return -1;
}

/* check block list to see if client's IP is banned */
int cs_is_blocked(int ip)
{
	int i;

	if (blocked_ips == NULL) {
		errno = EFAULT;
		return -1;
	}

	for (i = 0; i < MAX_IPBLOCKS; i++) {
		if (blocked_ips[i] == ip)
			return 1;
	}

	return 0;
}
